/*
  This file is a part of KMC software distributed under GNU GPL 3 licence.
  The homepage of the KMC project is http://sun.aei.polsl.pl/kmc

  Authors: Sebastian Deorowicz and Agnieszka Debudaj-Grabysz

  Version: 0.3
  Date   : 2012-11-29
*/

#ifndef _KB_COMPACTOR_H
#define _KB_COMPACTOR_H

#include "defs.h"
#include "params.h"
#include "kmer.h"
#include "radix.h"
#include <string>
#include <algorithm>
#include <numeric>
#include <array>
#include <stdio.h>

//************************************************************************************************************
// CKmerPackageCompactor - compact a package of bins and save to disk
//************************************************************************************************************
class CKmerPackageCompactor {
	CBinPackageQueue *q_package;
	CDiskMonitor *m_disk;
	CBinDesc *bd;
	CWriteBufferQueue **write_buffer_queue;
	CMemoryPool *pmm_bins, *pmm_cnts_small, *pmm_cnts_large;
	CMemoryMonitor *mm_write_buffer;

	uchar *buffer;
	uint64 buffer_size;
	int n_disks;

	void Compact(int32 bin_id, CBinPackageQueue::parts_t *package);
	void ReadPrefCounters(uint32 pref_size, uchar *buf, uint32 *cnts, uint32 &part_records);

public:
	CKmerPackageCompactor(CKMCParams &Params, CKMCQueues &Queues);
	~CKmerPackageCompactor();

	void ProcessQueue();
};

//************************************************************************************************************
// CKmerBinSorter - wrapper for multithreading purposes
//************************************************************************************************************
class CWKmerPackageCompactor {
	CKmerPackageCompactor *kpc;

public:
	CWKmerPackageCompactor(CKMCParams &Params, CKMCQueues &Queues);
	~CWKmerPackageCompactor();

	void operator()();
};

#endif

// ***** EOF
